@@ -1,4 +1,4 @@
-#include <slang/slang.h>
+#include <slang.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -30,7 +30,8 @@ static struct componentOps cbOps = {
     cbDraw,
     cbEvent,
     cbDestroy,
-    NULL,
+    newtDefaultPlaceHandler,
+    newtDefaultMappedHandler,
 } ;
 
 newtComponent newtListitem(int left, int top, const char * text, int isDefault,
@@ -105,6 +106,12 @@ newtComponent newtRadioGetCurrent(newtComponent setMember) {
     return setMember;
 }
 
+char newtCheckboxGetValue(newtComponent co) {
+    struct checkbox * cb = co->data;
+
+    return cb->value;
+}
+
 newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
 			   const char * seq, char * result) {
     newtComponent co;
