@@ -1,4 +1,4 @@
-#include <slang/slang.h>
+#include <slang.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +36,7 @@ static const char * defaultHelpLine =
 "  <Tab>/<Alt-Tab> between elements   |  <Space> selects   |  <F12> next screen"
 ;
 
-struct newtColors newtDefaultColorPalette = {
+const struct newtColors newtDefaultColorPalette = {
 	"white", "blue", 			/* root fg, bg */     
 	"black", "lightgray",			/* border fg, bg */
 	"black", "lightgray",			/* window fg, bg */
@@ -62,7 +62,7 @@ struct newtColors newtDefaultColorPalette = {
 	"black", "brown"			/* selected listbox */
 };
 
-static struct keymap keymap[] = {
+static const struct keymap keymap[] = {
 	{ "\033OA", 		NEWT_KEY_UP, 		"kh" },
 	{ "\033[A", 		NEWT_KEY_UP, 		"ku" },
 	{ "\033OB", 		NEWT_KEY_DOWN, 		"kd" },
@@ -82,6 +82,8 @@ static struct keymap keymap[] = {
 
 	{ "\033[5~",		NEWT_KEY_PGUP,		NULL },
 	{ "\033[6~",		NEWT_KEY_PGDN,		NULL },
+	{ "\033V",		NEWT_KEY_PGUP, 		"kH" },
+	{ "\033v",		NEWT_KEY_PGUP, 		"kH" },
 
 	{ "\033[[A",		NEWT_KEY_F1,		NULL },
 	{ "\033[[B",		NEWT_KEY_F2,		NULL },
@@ -262,7 +264,7 @@ void newtSetColors(struct newtColors colors) {
 int newtGetKey(void) {
     int key;
     char buf[10], * chptr = buf;
-    struct keymap * curr;
+    const struct keymap * curr;
 
     do {
 	key = SLang_getkey();
@@ -279,6 +281,14 @@ int newtGetKey(void) {
     } while (key == NEWT_KEY_SUSPEND);
 
     switch (key) {
+      case 'v' | 0x80:
+      case 'V' | 0x80:
+	return NEWT_KEY_PGUP;
+
+      case 22:
+	return NEWT_KEY_PGDN;
+
+	return NEWT_KEY_BKSPC;
       case 0x7f:
 	return NEWT_KEY_BKSPC;
 
@@ -618,3 +628,12 @@ void newtGetScreenSize(int * cols, int * rows) {
     if (rows) *rows = SLtt_Screen_Rows;
     if (cols) *cols = SLtt_Screen_Cols;
 }
+
+void newtDefaultPlaceHandler(newtComponent c, int newLeft, int newTop) {
+    c->left = newLeft;
+    c->top = newTop;
+}
+
+void newtDefaultMappedHandler(newtComponent c, int isMapped) {
+    c->isMapped = isMapped;
+}
