@@ -1,5 +1,5 @@
 #include <ctype.h>
-#include <slang/slang.h>
+#include <slang.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -13,22 +13,46 @@ struct textbox {
     int doWrap;
     newtComponent sb;
     int topLine;
+    int textWidth;
 };
 
-static void addLine(newtComponent co, const char * s, int len);
+static char * expandTabs(const char * text);
 static void textboxDraw(newtComponent co);
-static void addShortLine(newtComponent co, const char * s, int len);
+static void addLine(newtComponent co, const char * s, int len);
+static void doReflow(const char * text, char ** resultPtr, int width, 
+		     int * badness, int * heightPtr);
 static struct eventResult textboxEvent(newtComponent c,
 				      struct event ev);
 static void textboxDestroy(newtComponent co);
+static void textboxPlace(newtComponent co, int newLeft, int newTop);
+static void textboxMapped(newtComponent co, int isMapped);
 
 static struct componentOps textboxOps = {
     textboxDraw,
     textboxEvent,
     textboxDestroy,
-    NULL,
+    textboxPlace,
+    textboxMapped,
 } ;
 
+static void textboxMapped(newtComponent co, int isMapped) {
+    struct textbox * tb = co->data;
+
+    co->isMapped = isMapped;
+    if (tb->sb)
+	tb->sb->ops->mapped(tb->sb, isMapped);
+}
+
+static void textboxPlace(newtComponent co, int newLeft, int newTop) {
+    struct textbox * tb = co->data;
+
+    co->top = newTop;
+    co->left = newLeft;
+
+    if (tb->sb)
+	tb->sb->ops->place(tb->sb, co->left + co->width - 1, co->top);
+}
+
 void newtTextboxSetHeight(newtComponent co, int height) {
     co->height = height;
 }
@@ -39,6 +63,22 @@ int newtTextboxGetNumLines(newtComponent co) {
     return (tb->numLines);
 }
 
+newtComponent newtTextboxReflowed(int left, int top, char * text, int width,
+				  int flexDown, int flexUp, int flags) {
+    newtComponent co;
+    char * reflowedText;
+    int actWidth, actHeight;
+
+    reflowedText = newtReflowText(text, width, flexDown, flexUp,
+				  &actWidth, &actHeight);
+    
+    co = newtTextbox(-1, -1, actWidth, actHeight, NEWT_TEXTBOX_WRAP);
+    newtTextboxSetText(co, reflowedText);
+    free(reflowedText);
+
+    return co;
+}
+
 newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
     newtComponent co;
     struct textbox * tb;
@@ -53,29 +93,58 @@ newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
     co->top = top;
     co->left = left;
     co->takesFocus = 0;
+    co->width = width;
 
     tb->doWrap = flags & NEWT_FLAG_WRAP;
     tb->numLines = 0;
     tb->linesAlloced = 0;
     tb->lines = NULL;
     tb->topLine = 0;
+    tb->textWidth = width;
 
     if (flags & NEWT_FLAG_SCROLL) {
-	co->width = width - 2;
-	tb->sb = newtVerticalScrollbar(co->left + co->width + 1, co->top, 
+	co->width += 2;
+	tb->sb = newtVerticalScrollbar(co->left + co->width - 1, co->top, 
 			   co->height, COLORSET_TEXTBOX, COLORSET_TEXTBOX);
     } else {
-	co->width = width;
 	tb->sb = NULL;
     }
 
     return co;
 }
 
-static void doReflow(char * text, char ** resultPtr, int width, int * badness,
-		     int * heightPtr) {
+static char * expandTabs(const char * text) {
+    int bufAlloced = strlen(text) + 40;
+    char * buf, * dest;
+    const char * src;
+    int bufUsed = 0;
+    int i;
+
+    buf = malloc(bufAlloced + 1);
+    for (src = text, dest = buf; *src; src++) {
+	if ((bufUsed + 10) > bufAlloced) {
+	    bufAlloced += strlen(text) / 2;
+	    buf = realloc(buf, bufAlloced + 1);
+	    dest = buf + bufUsed;
+	}
+	if (*src == '\t') {
+	    i = 8 - (bufUsed & 8);
+	    memset(dest, ' ', i);
+	    dest += i, bufUsed += i;
+	} else {
+	    *dest++ = *src;
+	    bufUsed++;
+	}
+    }
+
+    *dest = '\0';
+    return buf;
+}
+
+static void doReflow(const char * text, char ** resultPtr, int width, 
+		     int * badness, int * heightPtr) {
     char * result = NULL;
-    char * chptr, * end;
+    const char * chptr, * end;
     int howbad = 0;
     int height = 0;
 
@@ -133,6 +202,9 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
     int i;
     char * result;
     int minbad, minbadwidth, howbad;
+    char * expandedText;
+
+    expandedText = expandTabs(text);
 
     if (flexDown || flexUp) {
 	min = width - flexDown;
@@ -142,7 +214,7 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
 	minbadwidth = width;
 
 	for (i = min; i <= max; i++) {
-	    doReflow(text, NULL, i, &howbad, NULL);
+	    doReflow(expandedText, NULL, i, &howbad, NULL);
 
 	    if (minbad == -1 || howbad < minbad) {
 		minbad = howbad;
@@ -153,7 +225,8 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
 	width = minbadwidth;
     }
 
-    doReflow(text, &result, width, NULL, actualHeight);
+    doReflow(expandedText, &result, width, NULL, actualHeight);
+    free(expandedText);
     if (actualWidth) *actualWidth = width;
     return result;
 }
@@ -161,74 +234,51 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
 void newtTextboxSetText(newtComponent co, const char * text) {
     const char * start, * end;
     struct textbox * tb = co->data;
+    char * reflowed, * expanded;
+    int badness, height;
 
     if (tb->lines) {
 	free(tb->lines);
-	tb->numLines = 0;
+	tb->linesAlloced = tb->numLines = 0;
     }
 
-    tb->linesAlloced = 10;
-    tb->lines = malloc(sizeof(char *) * 10);
+    expanded = expandTabs(text);
 
-    start = text;
+    if (tb->doWrap) {
+	doReflow(expanded, &reflowed, tb->textWidth, &badness, &height);
+	free(expanded);
+	expanded = reflowed;
+    }
+
+    for (start = expanded; *start; start++)
+	if (*start == '\n') tb->linesAlloced++;
+
+    /* This ++ leaves room for an ending line w/o a \n */
+    tb->linesAlloced++;
+    tb->lines = malloc(sizeof(char *) * tb->linesAlloced);
+
+    start = expanded;
     while ((end = strchr(start, '\n'))) {
 	addLine(co, start, end - start);
 	start = end + 1;
     }
 
     if (*start)
-	addLine(co, start, -1);
+	addLine(co, start, strlen(start));
+
+    free(expanded);
 }
 
-static void addLine(newtComponent co, const char * s, int origlen) {
-    const char * start, * end;
-    int len;
+/* This assumes the buffer is allocated properly! */
+static void addLine(newtComponent co, const char * s, int len) {
     struct textbox * tb = co->data;
 
-    if (origlen < 0) origlen = strlen(s);
-    len = origlen;
+    if (len > tb->textWidth) len = tb->textWidth;
 
-    if (!tb->doWrap || len <= co->width) {
-	addShortLine(co, s, len);
-    } else {
-	/* word wrap */
-
-	start = s;
-	while (len > co->width) {
-	    end = start + co->width - 1;
-	    while (end > start && !isspace(*end)) end--;
-
-	    if (end == start)
-		end = start + co->width - 1;
-
-	    addShortLine(co, start, end - start);
-	
-	    start = end + 1;
-	    while (isspace(*start) && *start) start++;
-
-	    len = origlen - (start - s);
-	}	
-
-	if (*start)
-	    addShortLine(co, start, len);
-    }
-}
-
-static void addShortLine(newtComponent co, const char * s, int len) {
-    struct textbox * tb = co->data;
-
-    if (tb->linesAlloced == tb->numLines) {
-	tb->linesAlloced += 10;
-	tb->lines = realloc(tb->lines, (sizeof(char *) * tb->linesAlloced));
-    }
-
-    if (len > co->width) len = co->width;
-
-    tb->lines[tb->numLines] = malloc(co->width + 1);
-    strncpy(tb->lines[tb->numLines], s, len);
-
-    while (len < co->width)  tb->lines[tb->numLines][len++] = ' ';
-    tb->lines[tb->numLines++][len] = '\0';
+    tb->lines[tb->numLines] = malloc(tb->textWidth + 1);
+    memset(tb->lines[tb->numLines], ' ', tb->textWidth); 
+    memcpy(tb->lines[tb->numLines], s, len);
+    tb->lines[tb->numLines++][tb->textWidth] = '\0';
 }
 
 static void textboxDraw(newtComponent c) {
@@ -270,6 +320,21 @@ static struct eventResult textboxEvent(newtComponent co,
 	    textboxDraw(co);
 	    er.result = ER_SWALLOWED;
 	    break;
+
+	  case NEWT_KEY_PGDN:
+	    tb->topLine += co->height;
+	    if (tb->topLine > (tb->numLines - co->height))
+		tb->topLine = tb->numLines - co->height;
+	    textboxDraw(co);
+	    er.result = ER_SWALLOWED;
+	    break;
+
+	  case NEWT_KEY_PGUP:
+	    tb->topLine -= co->height;
+	    if (tb->topLine < 0) tb->topLine = 0;
+	    textboxDraw(co);
+	    er.result = ER_SWALLOWED;
+	    break;
 	}
     }
 
